package it.inaf.oats.ia2.tapschemamanager.datalayer;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class Credentials implements Serializable {

    private static final long serialVersionUID = 1153912575502196261L;
    private static final Logger log = LoggerFactory.getLogger(Credentials.class);

    private String hostname;
    private int port;
    private String username;
    private String password;

    private DatabaseType databaseType;

    public Credentials() {
        super();
        this.port = 3306;
        this.databaseType = DatabaseType.MYSQL;
    }

    public String getHostname() {
        return this.hostname;
    }

    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    public int getPort() {
        return this.port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public String getUsername() {
        return this.username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return this.password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public DatabaseType getDatabaseType() {
        return databaseType;
    }

    public void setDatabaseType(DatabaseType databaseType) {
        this.databaseType = databaseType;
    }

    public String getDatabaseDriverClass() {
        switch (getDatabaseType()) {
            case MYSQL:
                return "com.mysql.jdbc.Driver";
        }
        return null;
    }

    public Connection getConnection() throws SQLException {
        try {
            Class.forName(getDatabaseDriverClass());
        } catch (ClassNotFoundException e) {
            e.printStackTrace(System.err);
        }

        String connectionString = "jdbc:mysql://" + this.hostname + ":" + this.port + "/";
        log.debug("Connecting to {} with username '{}'", connectionString, this.username);
        return DriverManager.getConnection(connectionString, this.username, this.password);
    }
}
