package it.inaf.oats.ia2.tapschemamanager.datalayer;

import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
@Entity
@Table(name = "`keys`")
public class KeyEntity implements Serializable {
    
    private static final long serialVersionUID = 4321901642780866919L;

    @Id
    @Column(name = "key_id", length = 64)
    private String keyId;

    @Column(name = "from_table", length = 128)
    private String fromTableFullName;

    @Column(name = "target_table", length = 128)
    private String targetTableFullName;

    @Column(name = "utype", length = 512)
    private String utype;

    @Column(name = "description", length = 512)
    private String description;

    @Column(name = "keyID")
    private BigInteger keyID;

    @ManyToOne(fetch = FetchType.EAGER)
    @PrimaryKeyJoinColumn(name = "from_table", referencedColumnName = "table_name")
    private TableEntity fromTable;

    @ManyToOne(fetch = FetchType.EAGER)
    @PrimaryKeyJoinColumn(name = "target_table", referencedColumnName = "table_name")
    private TableEntity targetTable;

    @OneToMany(mappedBy = "key", cascade = CascadeType.ALL, orphanRemoval = true)
    private final List<KeyColumnEntity> keyColumns;

    public KeyEntity() {
        keyColumns = new ArrayList<KeyColumnEntity>();
    }

    public KeyEntity(int keyId) {
        this();
        this.keyId = keyId + "";
    }

    public String getKeyId() {
        return keyId;
    }

    public void setKeyId(String keyId) {
        this.keyId = keyId;
    }

    public String getFromTableFullName() {
        return fromTableFullName;
    }

    public void setFromTableFullName(String fromTableFullName) {
        this.fromTableFullName = fromTableFullName;
    }

    public String getTargetTableFullName() {
        return targetTableFullName;
    }

    public void setTargetTableFullName(String targetTableFullName) {
        this.targetTableFullName = targetTableFullName;
    }

    public String getUtype() {
        return utype;
    }

    public void setUtype(String utype) {
        this.utype = utype;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public BigInteger getKeyID() {
        return keyID;
    }

    public void setKeyID(BigInteger keyID) {
        this.keyID = keyID;
    }

    public TableEntity getFromTable() {
        return fromTable;
    }

    public void setFromTable(TableEntity fromTable) {
        this.fromTable = fromTable;
    }

    public void setTargetTable(TableEntity targetTable) {
        this.targetTable = targetTable;
    }

    public TableEntity getTargetTable() {
        return targetTable;
    }

    public List<KeyColumnEntity> getKeyColumns() {
        return keyColumns;
    }
}
