package it.inaf.oats.ia2.tapschemamanager.datalayer;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public abstract class ResultSetReader {

    private static final Logger log = LoggerFactory.getLogger(ResultSetReader.class);

    private final String query;

    public ResultSetReader(String query) {
        this.query = query;
    }

    public abstract void manipulateItem(ResultSet resultSet) throws SQLException;

    public final void read(Connection connection) throws SQLException {
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            while (resultSet.next()) {
                manipulateItem(resultSet);
            }
        } catch (SQLException e) {
            log.error("The following query caused SQLException: \"" + query + "\"");
            throw e;
        } finally {
            if (resultSet != null) {
                resultSet.close();
            }
            if (statement != null) {
                statement.close();
            }
        }
    }
}
