
import it.inaf.oats.ia2.tapschemamanager.datalayer.ColumnEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.ColumnInfo;
import it.inaf.oats.ia2.tapschemamanager.datalayer.Credentials;
import it.inaf.oats.ia2.tapschemamanager.datalayer.DBWrapper;
import it.inaf.oats.ia2.tapschemamanager.datalayer.DatabaseType;
import it.inaf.oats.ia2.tapschemamanager.datalayer.KeyColumnEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.KeyEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.SchemaEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TableEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TapSchemaHandler;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import org.junit.Ignore;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class TestQuery {

    private static final int DATABASES_COUNT = 2;
    private static final int TABLES_COUNT = 3;

    private static List<DBWrapper> dbWrappers;

    public TestQuery() {
    }

    @BeforeClass
    public static void setUpClass() {

        dbWrappers = new ArrayList<>();

        Credentials credentials = new Credentials();
        credentials.setDatabaseType(DatabaseType.MYSQL);
        credentials.setHostname("localhost");
        credentials.setUsername("root");
        credentials.setPassword("root");

        //dbWrappers.add(new DBWrapper(credentials));
        credentials = new Credentials();
        credentials.setDatabaseType(DatabaseType.POSTGRES);
        credentials.setHostname("localhost");
        credentials.setUsername("postgres");
        credentials.setPassword("pippo");

        dbWrappers.add(new DBWrapper(credentials));
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    private void removeTestingDatabases() throws SQLException {
        for (DBWrapper dbWrapper : dbWrappers) {
            DatabaseType dbType = dbWrapper.getSourceDatabaseType();
            try (Connection connection = dbWrapper.getSourceConnection();
                    Statement statement = connection.createStatement()) {
                for (int i = 0; i < DATABASES_COUNT; i++) {
                    if (dbType == DatabaseType.MYSQL) {
                        statement.executeUpdate("DROP DATABASE IF EXISTS db" + i);
                    } else if (dbType == DatabaseType.POSTGRES) {
                        statement.executeUpdate("DROP SCHEMA IF EXISTS db" + i);
                    } else {
                        throw new UnsupportedOperationException("Database type " + dbType + " not supported");
                    }
                }
                if (dbType == DatabaseType.MYSQL) {
                    statement.executeUpdate("DROP DATABASE IF EXISTS test_tap_schema");
                } else if (dbType == DatabaseType.POSTGRES) {
                    statement.executeUpdate("DROP SCHEMA IF EXISTS test_tap_schema");
                } else {
                    throw new UnsupportedOperationException("Database type " + dbType + " not supported");
                }
            }
        }
    }

    private void setUpTestingDatabases() throws SQLException {
        removeTestingDatabases();

        for (DBWrapper dbWrapper : dbWrappers) {

            DatabaseType dbType = dbWrapper.getSourceDatabaseType();

            try (Connection connection = dbWrapper.getSourceConnection()) {

                try (Statement statement = connection.createStatement()) {

                    if (dbType == DatabaseType.MYSQL) {
                        for (int i = 0; i < DATABASES_COUNT; i++) {
                            statement.executeUpdate("CREATE DATABASE db" + i);
                            for (int j = 0; j < TABLES_COUNT; j++) {
                                statement.executeUpdate("CREATE TABLE db" + i + ".table" + j + " (\n"
                                        + "id INT PRIMARY KEY AUTO_INCREMENT,\n"
                                        + "value1 VARCHAR(255),\n"
                                        + "value2 FLOAT\n"
                                        + ");");
                            }
                        }
                    } else if (dbType == DatabaseType.POSTGRES) {
                        for (int i = 0; i < DATABASES_COUNT; i++) {
                            statement.executeUpdate("CREATE SCHEMA db" + i);
                            for (int j = 0; j < TABLES_COUNT; j++) {
                                statement.executeUpdate("CREATE TABLE db" + i + ".table" + j + " (\n"
                                        + "id BIGSERIAL PRIMARY KEY,\n"
                                        + "value1 VARCHAR(255),\n"
                                        + "value2 FLOAT\n"
                                        + ");");
                            }
                        }
                    } else {
                        throw new UnsupportedOperationException("Database type " + dbType + " not supported");
                    }
                }

                System.out.println("dbs created");

                connection.setCatalog("db0");

                statement = connection.createStatement();
                for (int j = 0; j < TABLES_COUNT - 1; j++) {
                    String update1 = "ALTER TABLE table" + (j + 1) + " ADD COLUMN table" + j + "_id INT";
                    statement.executeUpdate(update1);
                    //System.out.println(update1);
                    String update2 = "ALTER TABLE table" + (j + 1) + " ADD CONSTRAINT id_constraint_" + (j + 1) + " FOREIGN KEY(table" + j + "_id) REFERENCES table" + j + "(id)";
                    //System.out.println(update2);
                    statement.executeUpdate(update2);
                }
            } catch (SQLException e) {
                e.printStackTrace(System.err);
                throw e;
            } finally {
                if (statement != null) {
                    statement.close();
                }
                if (connection != null) {
                    connection.close();
                }
            }
        }
    }

    @Test
    public void testSerialization() throws Exception {
        DBWrapper dbWrapper = new DBWrapper(sourceCredentials, tapSchemaCredentials);

        TapSchemaHandler tapSchemaHandler = new TapSchemaHandler(dbWrapper, "tng_TAP_SCHEMA", true);

        System.out.println("schemas size = " + tapSchemaHandler.getSchemas().size());

        FileOutputStream fileOut = new FileOutputStream("/home/sonia/test.ser");
        ObjectOutputStream out = new ObjectOutputStream(fileOut);
        out.writeObject(tapSchemaHandler);
        out.close();
        fileOut.close();

        FileInputStream fileIn = new FileInputStream("/home/sonia/test.ser");
        ObjectInputStream in = new ObjectInputStream(fileIn);
        tapSchemaHandler = (TapSchemaHandler) in.readObject();
        in.close();
        fileIn.close();

        tapSchemaHandler.save();
        System.out.println("schemas size = " + tapSchemaHandler.getSchemas().size());
    }

    @Ignore
    @Test
    public void loadSchema() throws SQLException {
        DBWrapper dbWrapper = new DBWrapper(sourceCredentials, tapSchemaCredentials);
        TapSchemaHandler tapSchemaHandler = new TapSchemaHandler(dbWrapper, "tng_TAP_SCHEMA", true);
        assertTrue(!tapSchemaHandler.getSchemas().isEmpty());

        // Testing alphabetical order loading
        for (SchemaEntity schemaEntity : tapSchemaHandler.getSchemas()) {
            TableEntity previousTable = null;
            for (TableEntity table : schemaEntity.getTables()) {
                if (previousTable != null) {
                    //assertTrue("Table=" + table.getName() + ", Previous Table=" + previousTable.getName(),
                    //        previousTable.getName().compareToIgnoreCase(table.getName()) < 1);
                }
                previousTable = table;

                ColumnEntity previousColumn = null;
                for (ColumnEntity column : table.getColumns()) {
                    if (previousColumn != null) {
                        //assertTrue("Column=" + column.getName() + ", Previous Column=" + previousColumn.getName(),
                        //        previousColumn.getName().compareToIgnoreCase(column.getName()) < 1);
                    }
                    previousColumn = column;
                }

//                if(previousTable == null) {
//                    previousTable = table;
//                } else {
//                    
//                    
//                }
            }
        }

    }

    @Ignore
    @Test
    public void createNewAndUpdate() throws SQLException {
        System.out.println("TEST createNewAndUpdate STARTED");

        try {
            removeTestingDatabases(tapSchemaCredentials);

            setUpTestingDatabases(tapSchemaCredentials);

            // TEST first TapSchema creation
            DBWrapper dbWrapper = new DBWrapper(tapSchemaCredentials);
            TapSchemaHandler tsh = new TapSchemaHandler(dbWrapper, "test_tap_schema", false);

            SchemaEntity db0 = tsh.getNewSchema("db0");
            tsh.addSchema(db0);

            List<KeyEntity> db0Keys = tsh.getAllSchemaKeys("db0");
            assertEquals(db0Keys.size(), TABLES_COUNT - 1);
            // Print keys:
            for (KeyEntity schemaKey : db0Keys) {
                System.out.println(schemaKey.getFromTableFullName() + " -> " + schemaKey.getTargetTableFullName());
                for (KeyColumnEntity keyColumn : schemaKey.getKeyColumns()) {
                    System.out.println("\t" + schemaKey.getFromTableFullName() + "." + keyColumn.getFromColumn() + " -> " + schemaKey.getTargetTableFullName() + "." + keyColumn.getTargetColumn());
                }
            }

            // Test add, remove and re-add of a table
            //List<String> db0Tables = tsh.getAllTables("db0");
            TableEntity db0table0 = tsh.getNewTable("db0", "table0");
            assertEquals(db0table0.getName(), "db0.table0");
            tsh.addTable("db0", db0table0);
            for (ColumnInfo ci : tsh.getColumnInfo("db0", "table0")) {
                assertNotNull(ci.getColumnEntity().getTable());
            }

            TableEntity db0table1 = tsh.getNewTable("db0", "table1");
            tsh.addTable("db0", db0table1);
            assertEquals(db0.getTables().size(), 2);

            // Test key adding
            assertTrue(db0table0.getFromKeys().isEmpty());
            assertEquals(db0table0.getTargetKeys().size(), 1);

            assertTrue(db0table1.getTargetKeys().isEmpty());
            assertEquals(db0table1.getFromKeys().size(), 1);

            KeyEntity keyEntity = db0table1.getFromKeys().get(0);
            assertEquals(keyEntity.getFromTableFullName(), "db0.table1");
            assertEquals(keyEntity.getKeyColumns().get(0).getFromColumn(), "table0_id");
            assertEquals(keyEntity.getTargetTableFullName(), "db0.table0");
            assertEquals(keyEntity.getKeyColumns().get(0).getTargetColumn(), "id");

            TableEntity db0table2 = tsh.getNewTable("db0", "table2");
            tsh.addTable("db0", db0table2);
            assertEquals(db0table1.getFromKeys().size(), 1);
            assertEquals(db0table1.getTargetKeys().size(), 1);
            assertEquals(db0table2.getFromKeys().size(), 1);

            // Test key removing
            assertEquals(db0.getTables().size(), 3);
            tsh.removeTable(db0, db0table2);
            assertEquals(db0.getTables().size(), 2);
            assertEquals(db0table1.getFromKeys().size(), 1);
            assertTrue(db0table1.getTargetKeys().isEmpty());

            db0table2 = tsh.getNewTable("db0", "table2");
            tsh.addTable("db0", db0table2);
            assertEquals(db0table1.getFromKeys().size(), 1);
            assertEquals(db0table1.getTargetKeys().size(), 1);
            assertEquals(db0table2.getFromKeys().size(), 1);

            assertEquals(tsh.getForeignKeyReference("db0", "db0.table1", "table0_id"), "table0.id");
            assertNull(tsh.getForeignKeyReference("db0", "db0.table1", "id"));

            tsh.removeTable(db0, db0table1);
            assertTrue(db0table0.getFromKeys().isEmpty());
            assertTrue(db0table0.getTargetKeys().isEmpty());
            assertTrue(db0table2.getFromKeys().isEmpty());
            assertTrue(db0table2.getTargetKeys().isEmpty());

            // Test add column
            ColumnInfo ci = tsh.getColumnInfo("db0", "table1").get(0);
            ColumnEntity columnEntity = ci.getColumnEntity();
            tsh.addColumn(db0table1, columnEntity);
            assertNotNull(columnEntity.getTable());

            // Test save
            tsh.save();
            assertNotNull(db0.getTable("db0.table0"));

            // Test load saved
            tsh = new TapSchemaHandler(dbWrapper, "test_tap_schema", true);
            db0 = tsh.getSchemas().iterator().next();
            tsh.removeTable(db0, db0.getTable("db0.table0"));
            assertNull(db0.getTable("db0.table0"));
            tsh.save();

            tsh = new TapSchemaHandler(dbWrapper, "test_tap_schema", true);
            db0 = tsh.getSchemas().iterator().next();
            assertNull(db0.getTable("db0.table0"));

            // Test key constraints
            tsh.addTable("db0", db0table0);
            System.out.println("db0.table0 added");

//            assertEquals(db0Tables.size(), TABLES_COUNT);
//            for (String tableName : db0Tables) {
//                tapSchemaHandler.get
//            }
//            
//            TableEntity db0_
//            
//            List<String> addables = tapSchema.getAddables();
//            assertTrue(addables.contains("db0"));
//            assertTrue(addables.contains("db1"));
//
//            tapSchema.addEntityWrapper("db0");
//            assertFalse(tapSchema.getAddables().contains("db0"));
//
//            Schema schema = tapSchema.getSchema("db0");
//
//            addables = schema.getAddables();
//            for (int i = 0; i < addables.size(); i++) {
//                String tableName = addables.get(i);
//                assertEquals(tableName, "table" + i);
//                schema.addEntityWrapper(tableName);
//            }
//            assertTrue(schema.getAddables().isEmpty());
//
//            Table table1 = schema.getTable("table1");
//            assertEquals(table1.getName(), "table1");
//            assertEquals(table1.getFullName(), "db0.table1");
//            table1.addEntityWrapper("id");
//            table1.addEntityWrapper("value1");
//            assertFalse(table1.getAddables().contains("id"));
//            assertTrue(table1.getAddables().contains("value2"));
//
//            Column column = table1.getColumn("id");
//            column.setValue(Column.DESCRIPTION, "Test description");
//            assertTrue(column.isChanged());
//            column.setValue(Column.DESCRIPTION, "");
//            assertFalse(column.isChanged());
        } catch (SQLException e) {
            throw e;
        } finally {
            //removeTestingDatabases(credentials);
        }
    }

    @Ignore
    @Test
    public void query() throws SQLException {

//        EntityManagerFactory emf = Persistence.createEntityManagerFactory("pu");
//        EntityManager em = emf.createEntityManager();
        //CriteriaBuilder cb = em.getCriteriaBuilder();
        //CriteriaQuery<SchemaEntity> cq = cb.createQuery(SchemaEntity.class);
        //List<SchemaEntity> results = em.createQuery(cq).getResultList();
//        SchemaEntity schema = new SchemaEntity("test_schema");
//
//        TableEntity table = new TableEntity("table1");
//        schema.addTable(table);
//
//        ColumnEntity column = new ColumnEntity("table1_column1");
//        table.addColumn(column);
        //TapSchemaHandler tapSchemaHandler = new TapSchemaHandler(credentials, "TestJPA", false);
        //statement = connection.createStatement();
        //    String tapschemaName = tapSchema.getName();
        //creates database
        //String queryString = "CREATE DATABASE IF NOT EXISTS " + tapschemaName + ";";
        //statement.executeUpdate(queryString);
        //em.getTransaction().begin();
//        for (SchemaEntity schema : results) {
//            System.out.println("schema = " + schema.getSchemaName());
//            for (TableEntity table : schema.getTables().values()) {
//                System.out.println("table = " + table.getTableName());
//            }
//
////            if (schema.getSchemaName().equals("geo")) {
////                Map<String, TableEntity> tables = schema.getTables();
////                TableEntity table = schema.getTables().get("geo.country_block_ipv4");
////                //em.remove(table);
//////                for (ColumnEntity column : table.getColumns().values()) {
//////                    System.out.println("column = " + column.getColumnName());
//////                    column.setDescription("testDescription");
//////                }
////                break;
////            }
//        }
        //em.getTransaction().commit();
        //results.get(i)
//        for (SchemaEntity schema : results) {
//            System.out.println("schema = " + schema.getSchemaName());
//            for (TableEntity table : schema.getTables()) {
//                System.out.println("table = " + table.getTableName());
//                for (ColumnEntity column : table.getColumns()) {
//                    System.out.println("column = " + column.getColumnName());
//                }
//                for (KeyEntity fromKey : table.getFromKeys()) {
//                    System.out.println("fromKey = " + fromKey.getKeyId());
//                }
//                for (KeyEntity targetKey : table.getTargetKeys()) {
//                    System.out.println("targetKey = " + targetKey.getKeyId());
//                }
//            }
//        }
    }
}
